import logging
from django.core.management import BaseCommand
import sys
import traceback

from djangoautoconf.cmd_handler_base.database_connection_maintainer import DatabaseConnectionMaintainer
from djangoautoconf.management.commands.create_default_super_user import get_default_admin_username

__author__ = 'weijia'


class DjangoCmdBase(BaseCommand):

    LOG_LEVELS = ['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG']

    help = 'Help text here'
    # option_list = BaseCommand.option_list + (
    #     # make_option('--duration',
    #     #     action='store',
    #     #     dest='duration',
    #     #     type='int',
    #     #     default=0,
    #     #     help='Run task for this many seconds (0 or less to run forever) - default is 0'),
    #     # make_option('--sleep',
    #     #     action='store',
    #     #     dest='sleep',
    #     #     type='float',
    #     #     default=5.0,
    #     #     help='Sleep for this many seconds before checking for new tasks (if none were found) - default is 5'),
    #     make_option('--log-file',
    #                 action='store',
    #                 dest='log_file',
    #                 help='Log file destination'),
    #     make_option('--log-std',
    #                 action='store_true',
    #                 dest='log_std',
    #                 help='Redirect stdout and stderr to the logging system'),
    #     make_option('--log-level',
    #                 action='store',
    #                 type='choice',
    #                 choices=LOG_LEVELS,
    #                 dest='log_level',
    #                 help='Set logging level (%s)' % ', '.join(LOG_LEVELS)),
    # )

    def __init__(self, stdout=None, stderr=None, no_color=False):
        super(DjangoCmdBase, self).__init__(stdout, stderr, no_color)
        self.options = None

    def _configure_logging(self, log_level, log_file, log_std):

        if log_level:
            log_level = getattr(logging, log_level)

        config = {}
        if log_level:
            config['level'] = log_level
        if log_file:
            config['filename'] = log_file

        if config:
            logging.basicConfig(**config)

        if log_std:
            class StdOutWrapper(object):
                def write(self, s):
                    logging.info(s)

            class StdErrWrapper(object):
                def write(self, s):
                    logging.error(s)

            sys.stdout = StdOutWrapper()
            sys.stderr = StdErrWrapper()

    def handle(self, *args, **options):
        # logging.basicConfig(level=logging.DEBUG)
        # log_level = options.pop('log_level', None)
        # log_file = options.pop('log_file', None)
        # log_std = options.pop('log_std', False)
        self.options = options
        # duration = options.pop('duration', 0)
        # sleep = options.pop('sleep', 5.0)

        # self._configure_logging(log_level, log_file, log_std)

        # autodiscover()
        #
        # start_time = time.time()
        #
        # while (duration <= 0) or (time.time() - start_time) <= duration:
        #     if not tasks.run_next_task():
        #         logging.debug('waiting for tasks')
        #         time.sleep(sleep)
        if True:  # try:
            self.msg_loop()
        else:  # except:
            traceback.print_exc()

    # noinspection PyMethodMayBeStatic
    def get_username(self):
        return get_default_admin_username()

    def msg_loop(self):
        pass


class LongRunCmdBase(DjangoCmdBase, DatabaseConnectionMaintainer):
    pass



import copy
import importlib

__author__ = 'weijia'


def force_list(item_or_list):
    if (type(item_or_list) is list) or (type(item_or_list) is tuple):
        return item_or_list
    else:
        return [item_or_list]


class TemplateViewFactory(object):

    def __init__(self):
        super(TemplateViewFactory, self).__init__()
        self.__parent_class_list = []
        self.__form_class = None
        self.__model_class = None
        self.__generic_parent_class = None
        self.__exclude = []
        self.__operation = ""

    def set_generic_template_view(self, operation):
        self.__operation = operation
        generic_module = importlib.import_module("django.views.generic")
        view_class_name = "%sView" % operation
        view_class = getattr(generic_module, view_class_name)
        # parent_class_tuple = (ajax_mixin, AjaxableFormContextUpdateMixin, view_class)
        # self.parent_class_list.append(view_class)
        # self.parent_class_list.append(view_class)
        self.__generic_parent_class = view_class

    def set_form_class(self, form_class):
        self.__form_class = form_class

    def set_model_class(self, model_class):
        self.__model_class = model_class

    def add_parent_class(self, parent_class_or_array):
        if (type(parent_class_or_array) is list) or (type(parent_class_or_array) is tuple):
            # parent_classes = copy.copy(parent_class_or_array)
            # parent_classes.append(self.parent_class_list[0])
            # self.parent_class_list = parent_classes
            self.__parent_class_list.extend(parent_class_or_array)
        else:
            # self.parent_class_list.insert(0, parent_class_or_array)
            self.__parent_class_list.append(parent_class_or_array)

    def set_exclude(self, exclude):
        self.__exclude = exclude

    def get_view(self):
        class_attributes = {
            # "Meta": type("Meta", (), {"model": self.model_class, "fields": []}),
            "template_name": "form_view_base_template.html",
            "submit_button_text": self.__operation,
            "success_url": "../"
        }
        if not (self.__model_class is None):
            class_attributes["model"] = self.__model_class

        if len(self.__exclude) > 0:
            class_attributes["exclude"] = self.__exclude

        if not (self.__form_class is None):
            class_attributes["form_class"] = self.__form_class

        create_view_class = type(self.__get_target_class_name(),
                                 self.__get_parent_class_tuple(),
                                 class_attributes)
        return create_view_class

    def __get_target_class_name(self):
        return "%s%s%s" % (self.__model_class.__name__, self.__operation, "View")

    def __get_parent_class_tuple(self):
        parent_classes = self.__parent_class_list
        parent_classes.append(self.__generic_parent_class)
        return tuple(parent_classes)



from djangoautoconf.auth.req_auth_base_backend import ReqAuthBaeBackend
from djangoautoconf.django_utils import retrieve_param
from django.contrib.auth import authenticate, login


class UsernamePasswordBackend(ReqAuthBaeBackend):
    def authenticate(self, request):
        data = retrieve_param(request)
        if ("password" in data) and ("username" in data):
            user = authenticate(username=data["username"], password=data["password"])
            if user is not None:
                return user
        return None



import logging
try:
    from mysql_keys import username, password
except ImportError:
    username = "root"
    password = ""

logging.warn("Using mysql database")

DATABASES.update({
    'default': {
        'ENGINE': 'django.db.backends.mysql',  # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': MYSQL_DATABASE_NAME,                # Or path to database file if using sqlite3.
        'USER': username,                      # Not used with sqlite3.
        'PASSWORD': password,                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
})


from djangoautoconf.local_key_manager import ConfigurableAttributeGetter

__author__ = 'weijia'


INSTALLED_APPS += (
    'bootstrapform',
    'userenabootstrap',
    'userena',
    'social.apps.django_app.default',
    'guardian',
)


AUTHENTICATION_BACKENDS = (
    'userena.backends.UserenaAuthenticationBackend',
    'social.backends.open_id.OpenIdAuth',
    'social.backends.google.GoogleOpenId',
    'social.backends.google.GoogleOAuth2',
    'social.backends.google.GoogleOAuth',
    'social.backends.twitter.TwitterOAuth',
    'social.backends.yahoo.YahooOpenId',
    #'social.backends.weibo.WeiboOAuth2',
    #'social.backends.qq.QQOauth2',
    'social.backends.github.GithubOAuth2',
    'django.contrib.auth.backends.ModelBackend',
    'guardian.backends.ObjectPermissionBackend',
)

AUTH_PROFILE_MODULE = 'webmanager.MyProfile'

USERENA_SIGNIN_REDIRECT_URL = '/accounts/%(username)s/'
LOGIN_URL = '/accounts/signin/'
LOGOUT_URL = '/accounts/signout/'

#EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'

USERENA_ACTIVATION_REQUIRED = False
USERENA_SIGNIN_AFTER_SIGNUP = True

ANONYMOUS_USER_ID = -1


TEMPLATE_CONTEXT_PROCESSORS += (
    'social.apps.django_app.context_processors.backends',
    'social.apps.django_app.context_processors.login_redirect',
    "django.contrib.auth.context_processors.auth",
)

getter = ConfigurableAttributeGetter("weibo_api_keys", "webmanager")

weibo_api_key = getter.get_attr("weibo_api_key")
weibo_api_secret = getter.get_attr("weibo_api_secret")
SOCIAL_AUTH_WEIBO_KEY = weibo_api_key
SOCIAL_AUTH_WEIBO_SECRET = weibo_api_secret

getter = ConfigurableAttributeGetter("github_api_key", "webmanager")
github_api_key = getter.get_attr("github_api_key")
github_api_secret = getter.get_attr("github_api_secret")
SOCIAL_AUTH_GITHUB_KEY = github_api_key
SOCIAL_AUTH_GITHUB_SECRET = github_api_secret


SOCIAL_AUTH_WEIBO_DOMAIN_AS_USERNAME = True




from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import pre_save
from django.dispatch import receiver
from reversion.models import Version


def create_initial_version(obj):
    try:
        from reversion import default_revision_manager
        default_revision_manager.save_revision((obj,))
    except:
        try:
            from reversion.revisions import default_revision_manager
            default_revision_manager.save_revision((obj,))
        except:
            from reversion.revisions import add_to_revision
            add_to_revision(obj)


def get_versioned_item(content_type, pk):
    try:
        return Version.objects.filter(content_type=content_type).filter(object_id_int=pk)
    except:
        return Version.objects.filter(content_type=content_type).filter(object_id=pk)


global_save_signal_receiver = []


class PreSaveHandler(object):
    def __init__(self, model_inst):
        super(PreSaveHandler, self).__init__()
        self.model_inst = model_inst

    def object_save_handler(self, sender, instance, **kwargs):
        # logging.error("======================================")
        if not (instance.pk is None):
            content_type = ContentType.objects.get_for_model(self.model_inst)
            # versioned_pk_queryset = Version.objects.filter(content_type=content_type).filter(
            #     object_id_int=instance.pk)
            versioned_pk_queryset = get_versioned_item(content_type, instance.pk)
            if not versioned_pk_queryset.exists():
                item = self.model_inst.objects.get(pk=instance.pk)
                try:
                    create_initial_version(item)
                except:
                    pass


def add_reversion_before_save(model_inst):
    s = PreSaveHandler(model_inst)
    global_save_signal_receiver.append(s)
    receiver(pre_save, sender=model_inst)(s.object_save_handler)



__author__ = 'weijia'


INSTALLED_APPS += (
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    # ... include the providers you want to enable:
    'allauth.socialaccount.providers.amazon',
    'allauth.socialaccount.providers.angellist',
    'allauth.socialaccount.providers.asana',
    'allauth.socialaccount.providers.baidu',
    'allauth.socialaccount.providers.basecamp',
    'allauth.socialaccount.providers.bitbucket',
    'allauth.socialaccount.providers.bitbucket_oauth2',
    'allauth.socialaccount.providers.bitly',
    'allauth.socialaccount.providers.coinbase',
    # 'allauth.socialaccount.providers.daum',
    'allauth.socialaccount.providers.digitalocean',
    'allauth.socialaccount.providers.discord',
    'allauth.socialaccount.providers.douban',
    'allauth.socialaccount.providers.draugiem',
    'allauth.socialaccount.providers.dropbox',
    # 'allauth.socialaccount.providers.dropbox_oauth2',
    'allauth.socialaccount.providers.edmodo',
    'allauth.socialaccount.providers.eveonline',
    'allauth.socialaccount.providers.evernote',
    'allauth.socialaccount.providers.facebook',
    'allauth.socialaccount.providers.feedly',
    'allauth.socialaccount.providers.fivehundredpx',
    'allauth.socialaccount.providers.flickr',
    'allauth.socialaccount.providers.foursquare',
    'allauth.socialaccount.providers.fxa',
    'allauth.socialaccount.providers.github',
    'allauth.socialaccount.providers.gitlab',
    'allauth.socialaccount.providers.google',
    'allauth.socialaccount.providers.hubic',
    'allauth.socialaccount.providers.instagram',
    # 'allauth.socialaccount.providers.kakao',
    # 'allauth.socialaccount.providers.line',
    'allauth.socialaccount.providers.linkedin',
    'allauth.socialaccount.providers.linkedin_oauth2',
    'allauth.socialaccount.providers.mailru',
    # 'allauth.socialaccount.providers.naver',
    'allauth.socialaccount.providers.odnoklassniki',
    'allauth.socialaccount.providers.openid',
    'allauth.socialaccount.providers.orcid',
    'allauth.socialaccount.providers.paypal',
    'allauth.socialaccount.providers.persona',
    'allauth.socialaccount.providers.pinterest',
    'allauth.socialaccount.providers.reddit',
    'allauth.socialaccount.providers.robinhood',
    'allauth.socialaccount.providers.shopify',
    'allauth.socialaccount.providers.slack',
    'allauth.socialaccount.providers.soundcloud',
    'allauth.socialaccount.providers.spotify',
    'allauth.socialaccount.providers.stackexchange',
    'allauth.socialaccount.providers.stripe',
    'allauth.socialaccount.providers.tumblr',
    'allauth.socialaccount.providers.twentythreeandme',
    'allauth.socialaccount.providers.twitch',
    'allauth.socialaccount.providers.twitter',
    'allauth.socialaccount.providers.untappd',
    'allauth.socialaccount.providers.vimeo',
    'allauth.socialaccount.providers.vk',
    'allauth.socialaccount.providers.weibo',
    'allauth.socialaccount.providers.weixin',
    'allauth.socialaccount.providers.windowslive',
    'allauth.socialaccount.providers.xing',
)


AUTHENTICATION_BACKENDS += (
    # `allauth` specific authentication methods, such as login by e-mail
    'allauth.account.auth_backends.AuthenticationBackend',
)

SITE_ID = 1
